// author: allen @joslin .net

#include <stdlib.h> // for malloc and free

// eyes-on memory allocation 

uint8_t * heapptr, * stackptr;
void check_mem() {
	stackptr = (uint8_t *)malloc(4);  // use stackptr temporarily
	heapptr = stackptr;               // save value of heap pointer
	free(stackptr);      // free up the memory again (sets stackptr to 0)
	stackptr =  (uint8_t *)(SP);      // save value of stack pointer
}

void print_mem() { check_mem(); Serial.print(" mem:"); Serial.println((stackptr-heapptr),DEC); }

// ---------------------------------------------

void * operator new(size_t size) {  return malloc(size); }
void operator delete(void * ptr) {  if (ptr != NULL) { free(ptr); } }

// ---------------------------------------------

// Object PS2Keyboard :: this object is in charge of a PS2 keyboard

#include <PS2Keyboard.h>

// ---------------------------------------------

// Object SparkFunSerLCD :: this object is in charge of an LCD panel
//        these are 1-based devices, 1st cell is [1,1]

#include <SoftwareSerial.h>
#include <SparkFunSerLCD.h>

// --------------------------------------------

// Object LCDEcho :: this object is in charge of echoing keypresses from 
//     a PS2Keyboard to a SparkFunSerLCD

class LCDEcho {
private:
	int numRows, numCols, col, row;
	int lcdPin;

public:
	SparkFunSerLCD* lcd;
	PS2Keyboard* keys;

	LCDEcho ( int _data, int _clock, bool _lites, int _lcd, int _rows, int _cols  ) {
		keys = new PS2Keyboard(_data,_clock,_lites);
		lcd = new SparkFunSerLCD(_lcd,_rows,_cols);
		numRows = _rows; numCols = _cols; 
	}

	void start () {
		reset(); 
		lcd->setup(); 
		keys->begin();
	}

	void reset () {
		row = col = 1;
		lcd->empty();
	}

	inline bool available () { return keys->available(); }
	inline byte read () { return keys->read(); }

	void echo ( byte dat ) {

		if (dat < 0x80) {
			col++; lcd->at(row,col,(char)dat);
			if (col > numCols) {
				col = 1; 
				row++;
			}
		}

		if(dat == PS2_KC_BKSP) {
			lcd->at(row,col,' ');
			col--;
			if (col < 0) {
				col = numCols; 
				row--;
			}
		}

		if(dat == PS2_KC_ESC) {
			row = col = 1;
			reset();
		}

	}

};

// --------------------------------------------
// setup the testbed

LCDEcho* lcdEcho;

void setup () {
	Serial.begin(9600); 

	// LCDEcho ( int _PS2data, int _PS2clock, bool _PS2lites, int _lcdPin, int _rows, int _cols  ) 
	lcdEcho = new LCDEcho(2,3,false,10,2,16);

	lcdEcho->start();
}

// run the installation
void loop () {

	if (lcdEcho->available()) { 
		lcdEcho->echo(lcdEcho->read());
	}
}

// done 
// ---------------------------------------------------
