/* 
  Stepper Motor Control
  modified version of Tom Igoe's code with no library
  this is for a bipolar stepper motor:

  motor_pin_1 _____    ___    _____ motor_pin_3
                   )  /   \  (
                   ) |  M  | (
  motor_pin_2 _____)  \___/  (_____ motor_pin_4

  by jkCBWPnet
*/
  int spr = 200;		// change this to fit the number of steps per revolution
  int speed = 30;		// the motor speed, in revolutions per minute
  int motor_pin_1 = 3;		
  int motor_pin_2 = 4;		
  int motor_pin_3 = 18;		// set to 0 if 2 wire
  int motor_pin_4 = 19;		// set to 0 if 2 wire
  int pin_count = 4;		// 2 or 4 wire setup

  int stepCount = 0;		// number of steps the motor has taken
  int stepNumber = 0;		// which step the motor is on
  int last_step_time = 0;	// time stamp in ms of the last step taken
  int aMin = 60 * 1000;		// sec * ms/sec
  int sDelay = aMin/spr/speed;	// Sets the speed in revs per minute

// Moves the motor steps_to_move steps.
void step(int steps_to_move) {
  int direction;
  int steps_left = abs(steps_to_move);  // how many steps to take
  // determine direction based on whether steps_to_move is + or -:
  if (steps_to_move > 0) { direction = 1;}
  if (steps_to_move < 0) { direction = 0;}
  // decrement the number of steps, moving one step each time:
  while(steps_left > 0) {
  // move only if the appropriate delay has passed:
    if (millis()- last_step_time >= sDelay) {
      // step the motor to step number 0, 1, 2, or 3:
      stepMotor(stepNumber% 4);
      // get the timeStamp of when you stepped:
      last_step_time = millis();
      // increment or decrement the step number depending on direction:
      if (direction == 1) {
        stepNumber++;
        if (stepNumber == spr) {
          stepNumber = 0;
        }
      } else { 
        if (stepNumber == 0) {
          stepNumber = spr;
        }
        stepNumber--;
      }
      // decrement the steps left:
      steps_left--;
    }
  }
}          

// Actual controller output to the motor.
void stepMotor(int thisStep) {
  if (pin_count == 2) {
    switch (thisStep) {
      case 0: /* 01 */
        digitalWrite(motor_pin_1, LOW);
        digitalWrite(motor_pin_2, HIGH);
      break;
      case 1: /* 11 */
        digitalWrite(motor_pin_1, HIGH);
        digitalWrite(motor_pin_2, HIGH);
      break;
      case 2: /* 10 */
        digitalWrite(motor_pin_1, HIGH);
        digitalWrite(motor_pin_2, LOW);
      break;
      case 3: /* 00 */
        digitalWrite(motor_pin_1, LOW);
        digitalWrite(motor_pin_2, LOW);
      break;
    } 
  }
  if (pin_count == 4) {
    switch (thisStep) {
      case 0:    // 1010
        digitalWrite(motor_pin_1, HIGH);
        digitalWrite(motor_pin_2, LOW);
        digitalWrite(motor_pin_3, HIGH);
        digitalWrite(motor_pin_4, LOW);
      break;
      case 1:    // 0110
        digitalWrite(motor_pin_1, LOW);
        digitalWrite(motor_pin_2, HIGH);
        digitalWrite(motor_pin_3, HIGH);
        digitalWrite(motor_pin_4, LOW);
      break;
      case 2:    //0101
        digitalWrite(motor_pin_1, LOW);
        digitalWrite(motor_pin_2, HIGH);
        digitalWrite(motor_pin_3, LOW);
        digitalWrite(motor_pin_4, HIGH);
      break;
      case 3:    //1001
        digitalWrite(motor_pin_1, HIGH);
        digitalWrite(motor_pin_2, LOW);
        digitalWrite(motor_pin_3, LOW);
        digitalWrite(motor_pin_4, HIGH);
      break;
    } 
  }
}

void setup() {
  // initialize the stepper library on pins 8 through 11:
  pinMode(motor_pin_1, OUTPUT);
  pinMode(motor_pin_2, OUTPUT);
  pinMode(motor_pin_3, OUTPUT);
  pinMode(motor_pin_4, OUTPUT);

  // initialize the serial port:
  Serial.begin(115200);
}

void loop() {
  // step one step:
  step(200);
  Serial.print("steps:" );
  Serial.println(stepCount);
}

